#
# test_suite.py
#
# Copyright (c) 2017 Junpei Kawamoto
#
# This file is part of rgmining-tripadvisor-dataset.
#
# rgmining-tripadvisor-dataset is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# rgmining-tripadvisor-dataset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
"""Test suite.
"""
# pylint: disable=invalid-name
from __future__ import absolute_import, print_function
import importlib
import sys
import unittest


TESTS = (
    "tests.tripadvisor_test",
)
"""Collection of test modules."""


def suite():
    """Returns a test suite.
    """
    loader = unittest.TestLoader()
    res = unittest.TestSuite()

    for t in TESTS:
        mod = importlib.import_module(t)
        res.addTest(loader.loadTestsFromModule(mod))
    return res


def main():
    """The main function.

    Returns:
      Status code.
    """
    try:
        res = unittest.TextTestRunner(verbosity=2).run(suite())
    except KeyboardInterrupt:
        print("Test canceled.")
        return -1
    else:
        return 0 if res.wasSuccessful() else 1


if __name__ == "__main__":
    sys.exit(main())



#
# tripadvisor_test.py
#
# Copyright (c) 2017 Junpei Kawamoto
#
# This file is part of rgmining-tripadvisor-dataset.
#
# rgmining-tripadvisor-dataset is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# rgmining-tripadvisor-dataset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
"""Unit test for synthetic package.
"""
from collections import defaultdict
import random
import unittest
import tripadvisor


class GraphMock(object):
    """A mock of graph object.
    """

    def __init__(self):
        self.reviewers = set()
        self.products = set()
        self.reviews = defaultdict(dict)

    def new_reviewer(self, name):
        """Create a new reviewer.
        """
        if name in self.reviewers:
            raise ValueError("The given reviewer already exists:", name)
        self.reviewers.add(name)
        return name

    def new_product(self, name):
        """Create a new product.
        """
        if name in self.products:
            raise ValueError("The given product already exists:", name)
        self.products.add(name)
        return name

    def add_review(self, reviewer, product, score, _date=None):
        """Add a review.
        """
        if reviewer not in self.reviewers:
            raise ValueError("The given reviewer doesn't exist:", reviewer)
        if product not in self.products:
            raise ValueError("The given product doesn't exist:", product)
        self.reviews[reviewer][product] = score


class TestGraphMock(unittest.TestCase):
    """Test case for the mock object.
    """

    def setUp(self):
        """Set up for the tests.
        """
        self.graph = GraphMock()

    def test_new_reviewer(self):
        """If create same reviewers, mock should rises an error.
        """
        name = "test-reviewer"
        self.assertEqual(self.graph.new_reviewer(name), name)
        with self.assertRaises(ValueError):
            self.graph.new_reviewer(name)

    def test_new_product(self):
        """If create same products, mock should rises an error.
        """
        name = "test-product"
        self.assertEqual(self.graph.new_product(name), name)
        with self.assertRaises(ValueError):
            self.graph.new_product(name)

    def test_add_review(self):
        """Test add_review method.
        """
        reviewer = "test-reviewer"
        product = "test-product"
        self.graph.new_reviewer(reviewer)
        self.graph.new_product(product)

        score = random.random()
        self.graph.add_review(reviewer, product, score)
        self.assertEqual(self.graph.reviews[reviewer][product], score)

        with self.assertRaises(ValueError):
            self.graph.add_review(reviewer, reviewer, score)
        with self.assertRaises(ValueError):
            self.graph.add_review(product, product, score)


class TestLoad(unittest.TestCase):
    """Test case for load method.
    """

    def test_load(self):
        """Test load method.
        """
        graph = GraphMock()
        self.assertEqual(tripadvisor.load(graph), graph)

        for pmap in graph.reviews.values():
            for score in pmap.values():
                self.assertGreaterEqual(score, 0)
                self.assertLessEqual(score, 1)


if __name__ == "__main__":
    unittest.main()



#! /usr/bin/env python
#
# sample.py
#
# Copyright (c) 2017 Junpei Kawamoto
#
# This file is part of rgmining-tripadvisor-dataset.
#
# rgmining-tripadvisor-dataset is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# rgmining-tripadvisor-dataset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
"""Evaluate a review graph mining algorithm with the Trip Advisor dataset.
"""
# pylint: disable=invalid-name
from __future__ import absolute_import, division
import logging
from logging import getLogger
import sys

import dsargparse
import tripadvisor

LOGGER = getLogger(__name__)

#--------------------------
# Loading algorithms
#--------------------------
ALGORITHMS = {}
"""Dictionary of graph loading functions associated with installed algorithms.
"""

# Load and register RIA.
try:
    import ria
except ImportError:
    LOGGER.info("rgmining-ria is not installed.")
else:
    def ignore_args(func):
        """Returns a wrapped function which ignore given arguments."""
        def _(*_args):
            """The function body."""
            return func()
        return _
    ALGORITHMS["ria"] = ria.ria_graph
    ALGORITHMS["one"] = ignore_args(ria.one_graph)
    ALGORITHMS["onesum"] = ignore_args(ria.one_sum_graph)
    ALGORITHMS["mra"] = ignore_args(ria.mra_graph)


# Load and register RSD.
try:
    import rsd  # pylint: disable=wrong-import-position
except ImportError:
    LOGGER.info("rgmining-rsd is not installed.")
else:
    ALGORITHMS["rsd"] = rsd.ReviewGraph


# Load and register Fraud Eagle.
try:
    import fraud_eagle  # pylint: disable=wrong-import-position
except ImportError:
    LOGGER.info("rgmining-fraud-eagle is not installed.")
else:
    ALGORITHMS["feagle"] = fraud_eagle.ReviewGraph


# Load and register FRAUDAR.
try:
    import fraudar  # pylint: disable=wrong-import-position
except ImportError:
    LOGGER.info("rgmining-fraudar is not installed.")
else:
    def create_fraudar_graph(nblock=1):
        """Create a review graph defined in Fraud Eagle package.
        """
        return fraudar.ReviewGraph(int(nblock))
    ALGORITHMS["fraudar"] = create_fraudar_graph
#--------------------------


def run(method, loop, threshold, output, param):
    """Run a given algorithm with the Trip Advisor dataset.

    Runs a given algorithm and outputs anomalous scores and summaries after
    each iteration finishes. The function will ends if a given number of loops
    ends or the update of one iteration becomes smaller than a given threshold.

    Some algorithm requires a set of parameters. For example, feagle requires
    parameter `epsilon`. Argument `param` specifies those parameters, and
    if you want to set 0.1 to the `epsilon`, pass `epsilon=0.1` via the
    argument.

    Args:
      method: name of algorithm.
      loop: the number of iteration (default: 20).
      threshold: threshold to judge an update is negligible (default: 10^-3).
      output: writable object where the output will be written.
      param: list of key and value pair which are connected with "=".
    """
    kwargs = {key: float(value)
              for key, value in [v.split("=") for v in param]}
    g = ALGORITHMS[method](**kwargs)
    tripadvisor.load(g)

    tripadvisor.print_state(g, 0, output)

    # Updates
    logging.info("Start iterations.")
    for i in xrange(loop if not method.startswith("one") else 1):

        diff = g.update()
        if diff is not None and diff < threshold:
            break

        # Current summary
        logging.info("Iteration %d ends. (diff=%s)", i + 1, diff)
        tripadvisor.print_state(g, i + 1, output)

    # Print final state.
    tripadvisor.print_state(g, "final", output)


def main():
    """The main function.
    """
    if not ALGORITHMS:
        logging.error("No algorithms are installed.")
        sys.exit(1)

    parser = dsargparse.ArgumentParser(main=main)
    parser.add_argument("method", choices=sorted(ALGORITHMS.keys()))
    parser.add_argument(
        "--output", default=sys.stdout,
        type=dsargparse.FileType("w"),  # pylint: disable=no-member
        help="file path to store results (Default: stdout).")
    parser.add_argument("--loop", type=int, default=20)
    parser.add_argument("--threshold", type=float, default=10^-3)
    parser.add_argument(
        "--param", action="append", default=[],
        help=(
            "key and value pair which are connected with '='.\n"
            "This option can be set multiply."))

    run(**vars(parser.parse_args()))


if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO, stream=sys.stderr)
    try:
        main()
    except KeyboardInterrupt:
        pass
    except Exception:  # pylint: disable=broad-except
        logging.exception("Untracked exception occurred.")
    finally:
        logging.shutdown()



#
# tripadvisor.py
#
# Copyright (c) 2017 Junpei Kawamoto
#
# This file is part of rgmining-tripadvisor-dataset.
#
# rgmining-tripadvisor-dataset is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# rgmining-tripadvisor-dataset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# pylint: disable=invalid-name
"""This module provides a loading function of the Trip Advisor Dataset.

It also a helper function, :meth:`print_state`, to output a state of a graph
object.
To use both fuctions, the graph object must implement the
:ref:`graph interface <dataset-io:graph-interface>`.
"""
from __future__ import division
from contextlib import closing
import datetime
import json
from os.path import exists, join
import site
import sys
import tarfile


_DATE_FORMAT = "%B %d, %Y"
"""Data format in the dataset.
"""


def _files(tar):
    """Yields a file in the tar file.
    """
    info = tar.next()
    while info:
        if info.isfile():
            yield info
        info = tar.next()


def load(graph):
    """Load the Trip Advisor dataset to a given graph object.

    The graph object must implement the
    :ref:`graph interface <dataset-io:graph-interface>`.

    Args:
      graph: an instance of bipartite graph.

    Returns:
      The graph instance *graph*.
    """
    base = "TripAdvisorJson.tar.bz2"
    path = join(".", base)
    if not exists(path):
        path = join(sys.prefix, "rgmining","data", base)
    if not exists(path):
        path = join(sys.prefix, "local", "rgmining","data", base)
    if not exists(path):
        path = join(site.getuserbase(), "rgmining","data", base)

    R = {}  # Reviewers dict.
    with tarfile.open(path) as tar:

        for info in _files(tar):

            with closing(tar.extractfile(info)) as fp:

                obj = json.load(fp)

                target = obj["HotelInfo"]["HotelID"]
                product = graph.new_product(name=target)

                for r in obj["Reviews"]:
                    name = r["ReviewID"]
                    score = float(r["Ratings"]["Overall"]) / 5.

                    try:
                        date = datetime.datetime.strptime(
                            r["Date"], _DATE_FORMAT).strftime("%Y%m%d")
                    except ValueError:
                        date = None

                    if name not in R:
                        R[name] = graph.new_reviewer(name=name)
                    graph.add_review(R[name], product, score, date)

    return graph


def print_state(g, i, output=sys.stdout):
    """Print a current state of a given graph.

    This method outputs a current of a graph as a set of json objects.
    Graph objects must have two properties, `reviewers` and `products`.
    Those properties returns a set of reviewers and products respectively.
    See the :ref:`graph interface <dataset-io:graph-interface>`
    for more information.

    In this output format, each line represents a reviewer or product object.

    Reviewer objects are defined as ::

        {
           "iteration": <the iteration number given as i>
           "reviewer":
           {
              "reviewer_id": <Reviewer's ID>
              "score": <Anomalous score of the reviewer>
           }
        }

    Product objects are defined as ::

        {
           "iteration": <the iteration number given as i>
           "reviewer":
           {
              "product_id": <Product's ID>
              "sumarry": <Summary of the reviews for the product>
           }
        }

    Args:
      g: Graph instance.
      i: Iteration number.
      output: A writable object (default: sys.stdout).
    """
    for r in g.reviewers:
        json.dump({
            "iteration": i,
            "reviewer": {
                "reviewer_id": r.name,
                "score": r.anomalous_score
            }
        }, output)
        output.write("\n")

    for p in g.products:
        json.dump({
            "iteration": i,
            "product": {
                "product_id": p.name,
                "summary": float(str(p.summary))
            }
        }, output)
        output.write("\n")



#
# setup.py
#
# Copyright (c) 2017 Junpei Kawamoto
#
# This file is part of rgmining-tripadvisor-dataset.
#
# rgmining-tripadvisor-dataset is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# rgmining-tripadvisor-dataset is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
#
# pylint: disable=invalid-name
"""Package information about a synthetic dataset for review graph mining.
"""
import distutils.command.install_data
from os import path
import site
import sys
import urllib
from setuptools import setup


def read(fname):
    """Read a file.
    """
    return open(path.join(path.dirname(__file__), fname)).read()


class CustomInstallData(distutils.command.install_data.install_data):
    """Custom install data command to download data files from the web.
    """

    def run(self):
        """Before executing run command, download data files.
        """
        for f in self.data_files:
            if not isinstance(f, tuple):
                continue
            for i, u in enumerate(f[1]):
                base = path.basename(u)
                f[1][i] = path.join(sys.prefix, f[0], base)
                if not path.exists(f[1][i]):
                    f[1][i] = path.join(sys.prefix, "local", f[0], base)
                if not path.exists(f[1][i]):
                    f[1][i] = path.join(site.getuserbase(), f[0], base)
                if not path.exists(f[1][i]):
                    f[1][i] = urllib.urlretrieve(u, base)[0]
        return distutils.command.install_data.install_data.run(self)


def load_requires_from_file(filepath):
    """Read a package list from a given file path.

    Args:
      filepath: file path of the package list.

    Returns:
      a list of package names.
    """
    with open(filepath) as fp:
        return [pkg_name.strip() for pkg_name in fp.readlines()]


setup(
    name="rgmining-tripadvisor-dataset",
    use_scm_version=True,
    author="Junpei Kawamoto",
    author_email="kawamoto.junpei@gmail.com",
    description="Trip Advisor dataset for Review Graph Mining Project",
    long_description=read("README.rst"),
    url="https://github.com/rgmining/tripadvisor",
    py_modules=[
        "tripadvisor"
    ],
    install_requires=load_requires_from_file("requirements.txt"),
    setup_requires=[
        "setuptools_scm"
    ],
    data_files=[(
        "rgmining/data",
        ["http://times.cs.uiuc.edu/~wang296/Data/LARA/TripAdvisor/TripAdvisorJson.tar.bz2"]
    )],
    test_suite="tests.suite",
    license="GPLv3",
    cmdclass={
        "install_data": CustomInstallData
    },
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.6",
        "Topic :: Software Development :: Libraries",
        "Topic :: Scientific/Engineering :: Information Analysis"
    ]
)
